﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.Diagnostics;
using Ozeki.VoIP.SDK;
using Ozeki.VoIP;
using Ozeki.VoIP.Media;
using Ozeki.VoIP.SDP;
using Ozeki.VoIP.SIP;
using Ozeki.Common.Logger;
using Ozeki.Common;

namespace MessageManipulatorExample
{
    public partial class PhoneMain : Form
    {
        ISoftPhone softPhone;                   // The SoftPhone object
        SIPAccount sipAccount;                  // The SIP Account
        IPhoneLine phoneLine;                   // Represents a phone line
        IPhoneCall call;                        // The current phone call object
        PhoneLineState phoneLineState;          // Information about the state of the phone line
        bool inComingCall;                      // Indicates whether the current phone call is an incoming phone call
        SIPMessageLogger sipMessageLogger;      // Logs the sent and received SIP messages

        string localAddress = "192.168.1.2";    // local IP address
        string displayName = "Bob";             // SIP Account Display Name
        string userName = "Bob";                // SIP Account Username
        string authName = "Bob";                // SIP Account Authentication Username
        string password = "Bob123";             // SIP Account Password
        string domainHost = "sip.mysip.com";    // SIP Account Domain Server Host
        int domainPort = 5060;                  // SIP Account Domain Server Port Number

        SIP_CustomMessageManipulator sipMessageManipulator;
        SDP_CustomMessageManipulator sdpMessageManipulator;
        SDP_ConnectionManipulator connectionManipulator;
        SIP_OrganizationManipulator organizationManipulator;

        public PhoneMain()
        {
            InitializeComponent();
        }

        private void PhoneMain_Load(object sender, EventArgs e)
        {
            messagesTabControl.TabPages.Remove(organzationConnectionTabPage);
            messagesTabControl.TabPages.Remove(sdpTabPage);
            messagesTabControl.TabPages.Remove(sipTabPage);
            tsLabelOzeki.Text = string.Format("© Copyright {0}, Ozeki Informatics Ltd.", DateTime.Now.Year);

            InitializeSoftPhone();
            SetCustomSIPSDPTabPage();
            InvokeGUIThread(() =>
                {
                    tbPhoneLineStatus.Text = String.Format("{0} | Setup", sipAccount.DisplayName);
                });
        }

        private void PhoneMain_FormClosed(object sender, FormClosedEventArgs e)
        {
            if (softPhone != null)
                softPhone.Close();
        }

        #region Helper Functions
        /// <summary>
        ///It initializes a softphone object with a SIP BPX, and it is for requisiting a SIP account that is nedded for a SIP PBX service. It registers this SIP
        ///account to the SIP PBX through an ’IphoneLine’ object which represents the telephoneline. 
        ///If the telephone registration is successful we have a call ready softphone. In this example the softphone can be reached by dialing the number 891.
        /// </summary>
        private void InitializeSoftPhone()
        {
            try
            {
                sipAccount = new SIPAccount(true, displayName, userName, authName, password, domainHost, domainPort);
                softPhone = SoftPhoneFactory.CreateSoftPhone(SoftPhoneFactory.GetLocalIP(), 5700, 5750, 5700);
                softPhone.IncommingCall += new EventHandler<VoIPEventArgs<IPhoneCall>>(softPhone_IncomingCall);

                InitManipulators();

                phoneLine = softPhone.CreatePhoneLine(sipAccount);
                phoneLine.PhoneLineStateChanged += new EventHandler<VoIPEventArgs<PhoneLineState>>(phoneLine_PhoneLineStateChanged);

                sipMessageLogger = new SIPMessageLogger();
                sipMessageLogger.SIPMessageSent += new EventHandler<GenericEventArgs<string>>(sipMessageLogger_SIPMessageSent);
                sipMessageLogger.SIPMessageReceived += new EventHandler<GenericEventArgs<string>>(sipMessageLogger_SIPMessageReceived);
                Logger.Attach(sipMessageLogger);
                Logger.Open(LogLevel.Information);

                softPhone.RegisterPhoneLine(phoneLine);
            }
            catch (Exception ex)
            {
                MessageBox.Show(String.Format("You didn't give your local IP adress, so the program won't run properly.\n {0}", ex.Message), string.Empty, MessageBoxButtons.OK,
                MessageBoxIcon.Error);

                var sb = new StringBuilder();
                sb.AppendLine("Some error happened.");
                sb.AppendLine();
                sb.AppendLine("Exception:");
                sb.AppendLine(ex.Message);
                sb.AppendLine();
                if (ex.InnerException != null)
                {
                    sb.AppendLine("Inner Exception:");
                    sb.AppendLine(ex.InnerException.Message);
                    sb.AppendLine();
                }
                sb.AppendLine("StackTrace:");
                sb.AppendLine(ex.StackTrace);

                MessageBox.Show(sb.ToString());
            }
        }

        /// <summary>
        ///  It signs up to the necessary events of a call transact.
        /// </summary>
        private void WireUpCallEvents()
        {
            call.CallStateChanged += new EventHandler<VoIPEventArgs<CallState>>(call_CallStateChanged);
            call.CallErrorOccured += new EventHandler<VoIPEventArgs<CallError>>(call_CallErrorOccured);
        }

        /// <summary>
        /// It signs down from the necessary events of a call transact.
        /// </summary>
        private void WireDownCallEvents()
        {
            if (call != null)
            {
                call.CallStateChanged -= new EventHandler<VoIPEventArgs<CallState>>(call_CallStateChanged);
                call.CallErrorOccured -= new EventHandler<VoIPEventArgs<CallError>>(call_CallErrorOccured);
            }
        }

        /// <summary>
        /// The controls of the Windows Form applications can only be modified on the GUI thread. This method grants access to the GUI thread.
        /// </summary>
        /// <param name="action"></param>
        private void InvokeGUIThread(Action action)
        {
            Invoke(action);
        }
        #endregion

        #region Ozeki VoIP-SIP SDK's events

        /// <summary>
        /// Occurs when phone line state has changed.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void phoneLine_PhoneLineStateChanged(object sender, VoIPEventArgs<PhoneLineState> e)
        {
            phoneLineState = e.Item;
            string registerName = ((IPhoneLine) sender).SIPAccount.DisplayName;

            InvokeGUIThread(() =>
            {
                tbPhoneLineStatus.Text = String.Format("{0} | {1}", registerName, phoneLineState.ToString());
            });
        }

        /// <summary>
        /// Occurs when an incoming call request has received.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void softPhone_IncomingCall(object sender, VoIPEventArgs<IPhoneCall> e)
        {
            InvokeGUIThread(()=>
            {
                tbDial.Text = String.Format("Incoming call from {0}", e.Item.DialInfo);
                call = e.Item;
                WireUpCallEvents();
                inComingCall = true;
            });
        }

        /// <summary>
        /// Occurs when the phone call state has changed.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void call_CallStateChanged(object sender, VoIPEventArgs<CallState> e)
        {
            InvokeGUIThread(() => { tbDial.Text = String.Format("{0} | {1}", call.DialInfo, e.Item.ToString()); });

            switch (e.Item)
            {
                case CallState.Completed:
                    WireDownCallEvents();
                    call = null;

                    InvokeGUIThread(() => { tbDial.Text = string.Empty; });
                    break;
                case CallState.Cancelled:
                    WireDownCallEvents();
                    call = null;
                    break;
            }
        }

        /// <summary>
        /// There are certain situations when the call cannot be created, for example the dialed number is not available 
        /// or maybe there is no endpoint to the dialed PBX, or simply the telephone line is busy. 
        /// This event handling is for displaying these events.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void call_CallErrorOccured(object sender, VoIPEventArgs<CallError> e)
        {
            InvokeGUIThread(()=>
            {
                tbDial.Text = e.Item.ToString();
            });
        }

        #endregion

        #region Menu Events

        private void exitToolStripMenuItem_Click(object sender, EventArgs e)
        {
            Application.Exit();
        }

        private void openOnlineDocumentationToolStripMenuItem_Click(object sender, EventArgs e)
        {
            try
            {
                Process.Start("http://www.voip-sip-sdk.com/");
            }
            catch (Exception exception)
            {
                Debug.WriteLine(exception);
            }
        }

        private void aboutToolStripMenuItem_Click(object sender, EventArgs e)
        {
            AboutBox aboutBox = new AboutBox();

            aboutBox.ShowDialog(this);
        }

        #endregion

        #region Keypad Events

        /// <summary>
        /// It makes the dialing number.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void KeyPadButton_Click(object sender, EventArgs e)
        {
            var btn = sender as Button;
            if (btn == null)
                return;

            InvokeGUIThread(() =>
                {
                    if (btn.Text == "0/+")
                    {
                        tbDial.Text += "0";
                    }
                    else
                        tbDial.Text += btn.Text.Trim();
                });
        }

        private void btnPickUp_Click(object sender, EventArgs e)
        {
            if (inComingCall)
            {
                inComingCall = false;
                call.Accept();
                return;
            }

            if (call != null)
                return;

            if (string.IsNullOrEmpty(tbDial.Text))
                return;

            if (phoneLineState != PhoneLineState.RegistrationSucceded && phoneLineState != PhoneLineState.NoRegNeeded)
            {
                MessageBox.Show("Phone line state is not valid!");
                return;
            }

            call = softPhone.CreateCallObject(phoneLine, tbDial.Text);
            WireUpCallEvents();
            call.Start();
        }

        private void btnHangUp_Click(object sender, EventArgs e)
        {
            if (call != null)
            {
                inComingCall = false;
                call.HangUp();
                call = null;
            }

            InvokeGUIThread(() => { tbDial.Text = string.Empty; });
        }

        #endregion

        #region Manipulator Events

        /// <summary>
        /// Inizializes the SIP/SDP manipulator instances.
        /// </summary>
        private void InitManipulators()
        {
            connectionManipulator = new SDP_ConnectionManipulator();
            organizationManipulator = new SIP_OrganizationManipulator();
            sdpMessageManipulator = new SDP_CustomMessageManipulator();
            sipMessageManipulator = new SIP_CustomMessageManipulator();
        }

        /// <summary>
        /// Sets the "Connection" / "Organization" header manipulators.
        /// </summary>
        private void SetOrgConnManipulators()
        {
            if (softPhone != null)
            {
                softPhone.SetSIPMessageManipulator(organizationManipulator);
                softPhone.SetSDPMessageManipulator(connectionManipulator);
            }          
        }

        /// <summary>
        /// Sets the SIP/SDP message manipulators.
        /// </summary>
        private void SetCustomMessageManipulators()
        {
            if (softPhone != null)
            {
                softPhone.SetSIPMessageManipulator(sipMessageManipulator);
                softPhone.SetSDPMessageManipulator(sdpMessageManipulator);
            }
        }

        private void btnSIPApplyOutgoing_Click(object sender, EventArgs e)
        {
            sipMessageManipulator.CustomOutgoingMessage = tbOutgoingSIP.Text;
        }

        private void btnSIPApplyIncoming_Click(object sender, EventArgs e)
        {
            sipMessageManipulator.CustomIncomingMessage = tbIncomingSIP.Text;
        }

        private void btnSDPApplyOutgoing_Click(object sender, EventArgs e)
        {
            sdpMessageManipulator.CustomOutgoingMessage = tbOutgoingSDP.Text;
        }

        private void btnSDPApplyIncoming_Click(object sender, EventArgs e)
        {
            sdpMessageManipulator.CustomIncomingMessage = tbIncomingSDP.Text;
        }

        private void btnApplyOrganzation_Click(object sender, EventArgs e)
        {
            organizationManipulator.CustomInValue = tbIncomingOrganization.Text;
            organizationManipulator.CustomOutValue = tbOutgoingOrganization.Text;
        }

        private void btnConnectionApply_Click(object sender, EventArgs e)
        {
            connectionManipulator.CustomInValue = tbIncomingConnection.Text;
            connectionManipulator.CustomOutValue = tbOutgoingConnection.Text;
        }

        #endregion

        #region Message Logging

        /// <summary>
        /// Adds SIP message to the SIP message log.
        /// </summary>
        /// <param name="message">The SIP message.</param>
        public void SIPMessageReceived(string message)
        {
            if (InvokeRequired)
            {
                BeginInvoke((Action<PhoneMain, string>)((t, e1) => t.SIPMessageReceived(e1)), this, message);
                return;
            }

            SIPMessageTextBoxUpdate(message);
        }

        /// <summary>
        /// Adds SIP message to the SIP message log.
        /// </summary>
        /// <param name="message">The SIP message.</param>
        public void SIPMessageSent(string message)
        {
            if (InvokeRequired)
            {
                BeginInvoke((Action<PhoneMain, string>)((t, e1) => t.SIPMessageSent(e1)), this, message);
                return;
            }

            SIPMessageTextBoxUpdate(message);
        }

        /// <summary>
        /// Add SIP message string at the end of the SIP message text box, and scrolls there.
        /// </summary>
        /// <param name="text">The SIP message.</param>
        /// <remarks>
        /// If the containment is larger than 1MB the text is splitted to half.
        /// </remarks>
        private void SIPMessageTextBoxUpdate(string text)
        {
            string t = tbMessageLogs.Text;
            if (t.Length > 1024 * 1024)
                t = t.Substring(512 * 1024);

            tbMessageLogs.Text = t + text;
            tbMessageLogs.SelectionStart = tbMessageLogs.Text.Length;
            tbMessageLogs.ScrollToCaret();
            tbMessageLogs.Refresh();
        }

        /// <summary>
        /// Occurs when a SIP message sent through the network.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void sipMessageLogger_SIPMessageSent(object sender, GenericEventArgs<string> e)
        {
            SIPMessageSent(e.Item);
        }

        /// <summary>
        /// Occurs when a SIP message received.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void sipMessageLogger_SIPMessageReceived(object sender, GenericEventArgs<string> e)
        {
            SIPMessageReceived(e.Item);
        }

        #endregion

        #region CheckBox events

        private void radioCustomSIPSDP_CheckedChanged(object sender, EventArgs e)
        {
            if (radioCustomSIPSDP.Checked)
                SetCustomSIPSDPTabPage();
        }

        private void radioOrgConn_CheckedChanged(object sender, EventArgs e)
        {
            if (radioOrgConn.Checked)
                SetOrgConnTabPage();
        }

        private void SetOrgConnTabPage()
        {
            messagesTabControl.TabPages.Insert(0, organzationConnectionTabPage);
            messagesTabControl.TabPages.Remove(sdpTabPage);
            messagesTabControl.TabPages.Remove(sipTabPage);
            messagesTabControl.SelectedIndex = 0;
            SetOrgConnManipulators();
        }

        private void SetCustomSIPSDPTabPage()
        {
            messagesTabControl.TabPages.Remove(organzationConnectionTabPage);
            messagesTabControl.TabPages.Insert(0, sdpTabPage);
            messagesTabControl.TabPages.Insert(0, sipTabPage);
            messagesTabControl.SelectedIndex = 0;
            SetCustomMessageManipulators();
        }

        #endregion
    }
}
